<?php
/**
 * Zikula Application Framework
 *
 * @copyright (c) 2002, Zikula Development Team
 * @link http://www.zikula.org
 * @version $Id: displayfeed.php 33 2008-12-23 23:31:08Z Guite $
 * @license GNU/GPL - http://www.gnu.org/copyleft/gpl.html
 * @package Zikula_Value_Addons
 * @subpackage Feeds
 */

/**
 * initialise block
 */
function feeds_displayfeedblock_init()
{
    // Security
    pnSecAddSchema('Feeds:NewsFeed:', 'Block title::');
}

/**
 * get information on block
 */
function feeds_displayfeedblock_info()
{
    // Values
    return array('text_type' => 'displayfeed',
                 'module' => 'Feeds',
                 'text_type_long' => 'Show News Feed',
                 'allow_multiple' => true,
                 'form_content' => false,
                 'form_refresh' => false,
                 'show_preview' => true,
                 'admin_tableless' => true);
}

/**
 * display block
 */
function feeds_displayfeedblock_display($blockinfo)
{
    // Security check
    if (!SecurityUtil::checkPermission(
                         'Feeds:NewsFeed:',
                         "$blockinfo[title]::",
                         ACCESS_READ)) {
        return;
    }

    // Get variables from content block
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['feedid'])) {
        $vars['feedid'] = 1;
    }
    if (empty($vars['displayimage'])) {
        $vars['displayimage'] = 0;
    }
    if (empty($vars['displaydescription'])) {
        $vars['displaydescription'] = 0;
    }
    if (empty($vars['alternatelayout'])) {
        $vars['alternatelayout'] = 0;
    }

    // Get the feed item
    $item = pnModAPIFunc('Feeds', 'user', 'get', array('fid' => $vars['feedid']));

    if ($item == false) {
        return;
    }

    // Create output object
    $pnRender = pnRender::getInstance('Feeds');

    //  Check if the block is cached
    if ($pnRender->is_cached('feeds_block_displayfeed.htm', $item['fid'])) {
        $blockinfo['content'] = $pnRender->fetch('feeds_block_displayfeed.htm', $item['fid']);
        return pnBlockThemeBlock($blockinfo);
    }

    // Get the feed source
    $fullfeed = pnModAPIFunc('Feeds', 'user', 'getfeed', array('furl' => $item['url']));

    // Assign the module vars
    $pnRender->assign(pnModGetVar('Feeds'));

    // Assign the item and feed
    $pnRender->assign($item);
    $pnRender->assign('feed', $fullfeed);

    // assign the block vars
    $pnRender->assign($vars);

   // Populate block info and pass to theme
    $blockinfo['content'] = $pnRender->fetch('feeds_block_displayfeed.htm', $item['fid']);

    return pnBlockThemeBlock($blockinfo);
}

/**
 * modify block settings
 */
function feeds_displayfeedblock_modify($blockinfo)
{
    // Create output object
    $pnRender = pnRender::getInstance('Feeds', false);

    // Get current content
    $vars = pnBlockVarsFromContent($blockinfo['content']);

    // Defaults
    if (empty($vars['feedid'])) {
        $vars['feedid'] = 1;
    }
    if (empty($vars['displayimage'])) {
        $vars['displayimage'] = 0;
    }
    if (empty($vars['displaydescription'])) {
        $vars['displaydescription'] = 0;
    }
    if (empty($vars['alternatelayout'])) {
        $vars['alternatelayout'] = 0;
    }
    if (empty($vars['numitems'])) {
        $vars['numitems'] = -1;
    }

    // The API function is called.  The arguments to the function are passed in
    // as their own arguments array
    $items = pnModAPIFunc('Feeds', 'user', 'getall');

    // create an array for feednames and id's for the template
    $allfeeds = array();
    foreach($items as $item) {
        $allfeeds[$item['fid']] = $item['name'];
    }
    $pnRender->assign('allfeeds', $allfeeds);

    // assign the block vars
    $pnRender->assign($vars);

    // Return output
    return $pnRender->fetch('feeds_block_displayfeed_modify.htm');
}

/**
 * update block settings
 */
function feeds_displayfeedblock_update($blockinfo)
{
    $vars['feedid'] = FormUtil::getPassedValue('feedid', 1, 'POST');
    $vars['numitems'] = FormUtil::getPassedValue('numitems', 0, 'POST');
    $vars['displayimage'] = FormUtil::getPassedValue('displayimage', -1, 'POST');
    $vars['displaydescription'] = FormUtil::getPassedValue('displaydescription', -1, 'POST');
    $vars['alternatelayout'] = FormUtil::getPassedValue('alternatelayout', -1, 'POST');

    $blockinfo['content'] = pnBlockVarsToContent($vars);

    return $blockinfo;
}
